# -*- coding: utf-8 -*-
"""
    django-notifications
    ~~~~~
    A GitHub notification alike app for Django.
    :copyright: (c) 2015 by django-notifications team.
    :license: BSD, see LICENSE.txt for more details.
"""

# PEP 386-compliant version number: N.N[.N]+[{a|b|c|rc}N[.N]+][.postN][.devN]
__version__ = '1.0.0'

default_app_config = 'notifications.apps.Config'  # pylint: disable=invalid-name
