# Generated by Django 4.2.14 on 2024-07-24 09:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Customer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=64)),
                ("address", models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name="TargetObject",
            fields=[
                (
                    "customer_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="test_models.customer",
                    ),
                ),
            ],
            bases=("test_models.customer",),
        ),
    ]
